import React, { useMemo, useRef, useState, useEffect, forwardRef } from 'react';
import { ResizeObserver } from '@juggle/resize-observer';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _taggedTemplateLiteralLoose(strings, raw) {
  if (!raw) {
    raw = strings.slice(0);
  }

  strings.raw = raw;
  return strings;
}

function createHash(str) {
  var n = 0x5bd1e995;
  var l = str.length,
      h = l ^ l,
      i = 0,
      k;

  while (l >= 4) {
    k = str.charCodeAt(i) & 0xff | (str.charCodeAt(++i) & 0xff) << 8 | (str.charCodeAt(++i) & 0xff) << 16 | (str.charCodeAt(++i) & 0xff) << 24;
    k = (k & 0xffff) * n + (((k >>> 16) * n & 0xffff) << 16);
    k ^= k >>> 24;
    k = (k & 0xffff) * n + (((k >>> 16) * n & 0xffff) << 16);
    h = (h & 0xffff) * n + (((h >>> 16) * n & 0xffff) << 16) ^ k;
    l -= 4;
    ++i;
  }

  switch (l) {
    case 3:
      h ^= (str.charCodeAt(i + 2) & 0xff) << 16;

    case 2:
      h ^= (str.charCodeAt(i + 1) & 0xff) << 8;

    case 1:
      h ^= str.charCodeAt(i) & 0xff;
      h = (h & 0xffff) * 0x5bd1e995 + (((h >>> 16) * 0x5bd1e995 & 0xffff) << 16);
  }

  h ^= h >>> 13;
  h = (h & 0xffff) * n + (((h >>> 16) * n & 0xffff) << 16);
  h ^= h >>> 15;
  return (h >>> 0).toString(36);
}

var externalStyled = /*#__PURE__*/function () {
  try {
    require.resolveWeak('styled-components');

    return require('styled-components');
  } catch (_unused) {}

  try {
    require.resolveWeak('@emotion/styled');

    return require('@emotion/styled');
  } catch (_unused2) {}
}();

function createLayoutClassname(name, styles) {
  return "layout-block-" + name + "-" + createHash(styles);
}

function enhancePropsWithClassname(props, newClass) {
  var _props$className = props.className,
      className = _props$className === void 0 ? '' : _props$className;
  return _extends(_extends({}, props), {}, {
    className: className.length ? className.concat(" " + newClass) : newClass
  });
}

function processStyles(props, strings) {
  for (var _len = arguments.length, values = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    values[_key - 2] = arguments[_key];
  }

  return strings.map(function (str, i) {
    var valueType = typeof values[i];
    var value = valueType === 'function' ? values[i](props) : values[i] || '';
    return "" + str + value;
  }).join('').replace(/\s\s+/g, ' ');
}

function createStyleTag(styles) {
  return React.createElement("style", {
    dangerouslySetInnerHTML: {
      __html: styles
    }
  });
}

var _styled = {
  div: function div(strings) {
    for (var _len2 = arguments.length, values = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
      values[_key2 - 1] = arguments[_key2];
    }

    return React.forwardRef(function (props, ref) {
      var styles = processStyles.apply(void 0, [props, strings].concat(values));

      var _props$as = props.as,
          Tag = _props$as === void 0 ? 'div' : _props$as,
          _props$layoutName = props.layoutName,
          layoutName = _props$layoutName === void 0 ? 'box' : _props$layoutName,
          children = props.children,
          rest = _objectWithoutPropertiesLoose(props, ["as", "layoutName", "children"]);

      var layoutClass = createLayoutClassname(layoutName, styles);
      var selector = Tag + "." + layoutClass;
      return React.createElement(React.Fragment, null, createStyleTag(styles.replace(/&/g, selector)), React.createElement(Tag, Object.assign({}, enhancePropsWithClassname(_extends(_extends({}, rest), {}, {
        ref: ref
      }), layoutClass)), children));
    });
  }
};
var styled = externalStyled ? externalStyled["default"] : _styled;

function _templateObject() {
  var data = _taggedTemplateLiteralLoose(["\n  & {\n    width: 100vw;\n    margin-left: 50%;\n    transform: translateX(-50%);\n  }\n"]);

  _templateObject = function _templateObject() {
    return data;
  };

  return data;
}
/**
 * Breakout layout component
 */

var _Breakout = /*#__PURE__*/styled.div( /*#__PURE__*/_templateObject());

var Breakout = function Breakout(props) {
  return React.createElement(_Breakout, Object.assign({}, _extends(_extends({}, props), {}, {
    layoutName: 'breakout'
  })));
};

function _templateObject$1() {
  var data = _taggedTemplateLiteralLoose(["\n  & {\n    box-sizing: content-box;\n    margin-left: auto;\n    margin-right: auto;\n    max-width: ", ";\n    ", "\n  }\n"]);

  _templateObject$1 = function _templateObject() {
    return data;
  };

  return data;
}
/**
 * Center layout component
 */

var _Center = /*#__PURE__*/styled.div( /*#__PURE__*/_templateObject$1(), function (_ref) {
  var _ref$max = _ref.max,
      max = _ref$max === void 0 ? '60ch' : _ref$max;
  return max;
}, function (_ref2) {
  var _ref2$intrinsic = _ref2.intrinsic,
      intrinsic = _ref2$intrinsic === void 0 ? false : _ref2$intrinsic;
  return intrinsic ? "\n      display:flex;\n      flex-direction: column;\n      align-items: center;\n    " : '';
});

var Center = function Center(props) {
  return React.createElement(_Center, Object.assign({}, _extends(_extends({}, props), {}, {
    layoutName: 'center'
  })));
};

function _templateObject$2() {
  var data = _taggedTemplateLiteralLoose(["\n  & {\n    display: flex;\n    flex-direction: column;\n    min-height: ", ";\n  }\n"]);

  _templateObject$2 = function _templateObject() {
    return data;
  };

  return data;
}

var _Cover = /*#__PURE__*/styled.div( /*#__PURE__*/_templateObject$2(), function (_ref) {
  var _ref$height = _ref.height,
      height = _ref$height === void 0 ? '100vh' : _ref$height;
  return height;
});

var Cover = function Cover(props) {
  return React.createElement(_Cover, Object.assign({}, _extends(_extends({}, props), {}, {
    layoutName: 'cover'
  })));
};

function _templateObject$3() {
  var data = _taggedTemplateLiteralLoose(["\n  ", "\n\n  ", "\n\n  & > * {\n    overflow: hidden;\n    position: absolute;\n    top: 0;\n    right: 0;\n    bottom: 0;\n    left: 0;\n    display: flex;\n    justify-content: center;\n    align-items: center;\n  }\n\n  & > img,\n  & > video {\n    width: 100%;\n    height: 100%;\n    object-fit: cover;\n  }\n"]);

  _templateObject$3 = function _templateObject() {
    return data;
  };

  return data;
}

var _Frame = /*#__PURE__*/styled.div( /*#__PURE__*/_templateObject$3(), function (_ref) {
  var _ref$ratio = _ref.ratio,
      ratio = _ref$ratio === void 0 ? '16:9' : _ref$ratio;

  var _ratio$split = ratio.split(':'),
      denominator = _ratio$split[0],
      numerator = _ratio$split[1];

  return "\n    & {\n    padding-bottom: calc(" + Number(numerator) + " / " + Number(denominator) + " * 100%);\n    position: relative;\n  }\n    ";
}, function (_ref2) {
  var _ref2$overlay = _ref2.overlay,
      overlay = _ref2$overlay === void 0 ? '' : _ref2$overlay;
  return overlay ? "\n        &:after {\n          content: '';\n          position: absolute;\n          top: 0;\n          left: 0;\n          height: 100%;\n          width: 100%;\n          background-image: " + overlay + ";\n        }\n      " : '';
});

var Frame = function Frame(props) {
  return React.createElement(_Frame, Object.assign({}, _extends(_extends({}, props), {}, {
    layoutName: 'cover'
  })));
};

function useResizeObserver(opts) {
  if (opts === void 0) {
    opts = {};
  }

  var defaultRef = useRef(null);
  var onResize = opts.onResize;
  var onResizeRef = useRef(undefined);
  onResizeRef.current = onResize;
  var resizeObserverRef = useRef();
  var ref = defaultRef;

  var _useState = useState({
    width: 0,
    height: 0
  }),
      size = _useState[0],
      setSize = _useState[1];

  var previous = useRef({
    width: 0,
    height: 0
  });
  useEffect(function () {
    if (resizeObserverRef.current) return;
    resizeObserverRef.current = new ResizeObserver(function (entries) {
      if (!Array.isArray(entries)) {
        return;
      }

      if (!entries.length) {
        return;
      }

      var entry = entries[0];
      var _entry$borderBoxSize = entry.borderBoxSize,
          boxSize = _entry$borderBoxSize[0];
      var newWidth = Math.round(boxSize.inlineSize);
      var newHeight = Math.round(boxSize.blockSize);

      if (previous.current.width !== newWidth || previous.current.height !== newHeight) {
        var newSize = {
          width: newWidth,
          height: newHeight
        };

        if (onResizeRef.current) {
          onResizeRef.current(newSize);
        } else {
          previous.current.width = newWidth;
          previous.current.height = newHeight;
          setSize(newSize);
        }
      }
    });
  }, []);
  useEffect(function () {
    if (typeof ref !== 'object' || ref === null || !(ref.current instanceof Element)) {
      return;
    }

    var element = ref.current;
    resizeObserverRef.current.observe(element);
    return function () {
      return resizeObserverRef.current.unobserve(element);
    };
  }, [ref]);
  return useMemo(function () {
    return {
      ref: ref,
      width: size.width,
      height: size.height
    };
  }, [ref, size ? size.width : null, size ? size.height : null]);
}

function _templateObject$4() {
  var data = _taggedTemplateLiteralLoose(["\n  & {\n    display: grid;\n    grid-gap: ", ";\n    grid-template-columns: 100%;\n  }\n\n  &.aboveMin {\n    grid-template-columns: repeat(\n      auto-fill,\n      minmax(", ", 1fr)\n    );\n  }\n\n  @supports (width: min(", ", 100%)) {\n    & {\n      grid-template-columns: repeat(\n        auto-fit,\n        minmax(min(", ", 100%), 1fr)\n      );\n    }\n  }\n"]);

  _templateObject$4 = function _templateObject() {
    return data;
  };

  return data;
}
/**
 * Breakout layout component
 */

var _Grid = /*#__PURE__*/styled.div( /*#__PURE__*/_templateObject$4(), function (_ref) {
  var _ref$gap = _ref.gap,
      gap = _ref$gap === void 0 ? '0px' : _ref$gap;
  return gap;
}, function (_ref2) {
  var _ref2$min = _ref2.min,
      min = _ref2$min === void 0 ? '200px' : _ref2$min;
  return min;
}, function (_ref3) {
  var _ref3$min = _ref3.min,
      min = _ref3$min === void 0 ? '200px' : _ref3$min;
  return min;
}, function (_ref4) {
  var _ref4$min = _ref4.min,
      min = _ref4$min === void 0 ? '200px' : _ref4$min;
  return min;
});

var Grid = function Grid(props) {
  var _props$min = props.min,
      min = _props$min === void 0 ? '200px' : _props$min;
  var childRef = useRef(null);

  var _useResizeObserver = useResizeObserver(),
      ref = _useResizeObserver.ref,
      width = _useResizeObserver.width;

  var isWide = false; // Add a test element to compare the min value to the container

  if (childRef.current) {
    childRef.current.style.width = min;
    isWide = width > childRef.current.offsetWidth;
  }

  var containerClass = 'aboveMin';
  var _props$className = props.className,
      className = _props$className === void 0 ? '' : _props$className;

  var enhancedProps = _extends(_extends({}, props), {}, {
    layoutName: 'grid',
    ref: ref,
    className: className.length ? isWide ? className.concat(" " + containerClass) : className : isWide ? containerClass : className
  });

  return React.createElement(_Grid, Object.assign({}, enhancedProps), props.children, React.createElement("div", {
    ref: childRef
  }));
};

function _templateObject$5() {
  var data = _taggedTemplateLiteralLoose(["\n  & > * {\n    display: flex;\n    flex-wrap: wrap;\n    justify-content: ", ";\n    align-items: ", ";\n    margin: calc(", " / 2 * -1);\n  }\n\n  & > * > * {\n    margin: calc(", " / 2);\n  }\n"]);

  _templateObject$5 = function _templateObject() {
    return data;
  };

  return data;
}

var _Cluster = /*#__PURE__*/styled.div( /*#__PURE__*/_templateObject$5(), function (_ref) {
  var _ref$justify = _ref.justify,
      justify = _ref$justify === void 0 ? 'start' : _ref$justify;
  return ['start', 'end'].includes(justify) ? "flex-" + justify : justify;
}, function (_ref2) {
  var _ref2$align = _ref2.align,
      align = _ref2$align === void 0 ? 'stretch' : _ref2$align;
  return ['start', 'end'].includes(align) ? "flex-" + align : align;
}, function (_ref3) {
  var _ref3$gap = _ref3.gap,
      gap = _ref3$gap === void 0 ? '0px' : _ref3$gap;
  return gap;
}, function (_ref4) {
  var _ref4$gap = _ref4.gap,
      gap = _ref4$gap === void 0 ? '0px' : _ref4$gap;
  return gap;
});

var Cluster = function Cluster(props) {
  return React.createElement(_Cluster, Object.assign({}, _extends(_extends({}, props), {}, {
    layoutName: 'cluster'
  })), React.createElement("div", null, props.children));
};

function _templateObject$6() {
  var data = _taggedTemplateLiteralLoose(["\n  & {\n    position: absolute;\n    z-index: ", ";\n    ", "\n    ", "\n    ", "\n  }\n"]);

  _templateObject$6 = function _templateObject() {
    return data;
  };

  return data;
}

function generatePinningRules(x, y, margin) {
  return "\n    " + generateXPinningRules(x, margin) + "\n    " + generateYPinningRules(y, margin) + "\n    " + (x === 'center' && y === 'center' ? "transform: translate(-50%,-50%);" : x === 'center' ? "transform: translateX(-50%);" : y === 'center' ? "transform: translateY(-50%);" : '') + "\n  ";
}

function generateXPinningRules(x, margin) {
  if (x === 'left') {
    return "\n      left: " + margin + ";\n    ";
  }

  if (x === 'right') {
    return "\n      right: " + margin + ";\n    ";
  }

  if (x === 'full') {
    return "\n      left: " + margin + ";\n      right: " + margin + ";\n    ";
  }

  return "\n      left: 50%;\n    ";
}

function generateYPinningRules(y, margin) {
  if (y === 'top') {
    return "\n      top: " + margin + ";\n    ";
  }

  if (y === 'bottom') {
    return "\n      bottom: " + margin + ";\n    ";
  }

  if (y === 'full') {
    return "\n      top: " + margin + ";\n      bottom: " + margin + ";\n    ";
  }

  return "\n      top: 50%;\n    ";
}

var _Position = /*#__PURE__*/styled.div( /*#__PURE__*/_templateObject$6(), function (_ref) {
  var _ref$zIndex = _ref.zIndex,
      zIndex = _ref$zIndex === void 0 ? 'auto' : _ref$zIndex;
  return zIndex;
}, function (_ref2) {
  var _ref2$x = _ref2.x,
      x = _ref2$x === void 0 ? 'center' : _ref2$x,
      _ref2$y = _ref2.y,
      y = _ref2$y === void 0 ? 'center' : _ref2$y,
      _ref2$gutter = _ref2.gutter,
      gutter = _ref2$gutter === void 0 ? '0px' : _ref2$gutter;
  return generatePinningRules(x, y, gutter);
}, function (_ref3) {
  var _ref3$breakout = _ref3.breakout,
      breakout = _ref3$breakout === void 0 ? false : _ref3$breakout,
      _ref3$gutter = _ref3.gutter,
      gutter = _ref3$gutter === void 0 ? '0px' : _ref3$gutter;
  return !breakout ? "\n      max-width: calc(100% - (" + gutter + " * 2));\n      max-height: calc(100% - (" + gutter + " * 2));\n      overflow: auto;" : '';
}, function (_ref4) {
  var _ref4$fixed = _ref4.fixed,
      fixed = _ref4$fixed === void 0 ? false : _ref4$fixed;
  return fixed ? "\n      position: fixed;" : '';
});

var Position = function Position(props) {
  return React.createElement(_Position, Object.assign({}, _extends(_extends({}, props), {}, {
    layoutName: 'breakout'
  })));
};

function _templateObject$7() {
  var data = _taggedTemplateLiteralLoose(["\n  & {\n    display: flex;\n    flex-direction: ", ";\n    justify-content: ", ";\n    align-items: ", ";\n  }\n\n  & > *:not(style) ~ *:not(style) {\n    ", "\n  }\n"]);

  _templateObject$7 = function _templateObject() {
    return data;
  };

  return data;
}
var StyledStack = /*#__PURE__*/styled.div( /*#__PURE__*/_templateObject$7(), function (_ref) {
  var _ref$inline = _ref.inline,
      inline = _ref$inline === void 0 ? false : _ref$inline;
  return inline ? 'row' : 'column';
}, function (_ref2) {
  var _ref2$justify = _ref2.justify,
      justify = _ref2$justify === void 0 ? 'start' : _ref2$justify;
  return ['start', 'end'].includes(justify) ? "flex-" + justify : justify;
}, function (_ref3) {
  var _ref3$align = _ref3.align,
      align = _ref3$align === void 0 ? 'stretch' : _ref3$align;
  return ['start', 'end'].includes(align) ? "flex-" + align : align;
}, function (_ref4) {
  var _ref4$inline = _ref4.inline,
      inline = _ref4$inline === void 0 ? false : _ref4$inline,
      _ref4$gap = _ref4.gap,
      gap = _ref4$gap === void 0 ? '0px' : _ref4$gap;
  return "\n    margin-" + (inline ? 'left' : 'top') + ": " + gap + ";\n    ";
});
var Stack = /*#__PURE__*/forwardRef(function (props, ref) {
  return React.createElement(StyledStack, Object.assign({}, _extends(_extends({}, props), {}, {
    ref: ref,
    layoutName: 'stack'
  })));
});

console.log(require.resolve('react'));

function debounce(func, wait) {
  if (wait === void 0) {
    wait = 50;
  }

  var timeout;
  return [function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var context = this;

    var later = function later() {
      timeout = null;
      func.apply(context, args);
    };

    clearTimeout(timeout);
    timeout = setTimeout(later, wait);
  }, function () {
    return clearTimeout(timeout);
  }];
}

var useRefWidth = function useRefWidth() {
  for (var _len2 = arguments.length, breakpoints = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    breakpoints[_key2] = arguments[_key2];
  }

  var _useState = useState(0),
      width = _useState[0],
      setWidth = _useState[1];

  var _useState2 = useState(breakpoints.map(function () {
    return false;
  })),
      matches = _useState2[0],
      setMatches = _useState2[1];

  var _useMemo = useMemo(function () {
    return debounce(function (size) {
      setWidth(size.width);
    });
  }, [width]),
      onResize = _useMemo[0],
      clear = _useMemo[1];

  var _useResizeObserver = useResizeObserver({
    onResize: onResize
  }),
      ref = _useResizeObserver.ref;

  useEffect(function () {
    var el = document.createElement('div');
    el.style.position = 'absolute';
    if (ref.current) ref.current.appendChild(el);
    setMatches(breakpoints.map(function (breakpoint) {
      el.style.width = breakpoint;
      return width >= el.offsetWidth;
    }));
    return function () {
      el.remove();
      clear();
    };
  }, [width]);
  return useMemo(function () {
    return [ref].concat(matches);
  }, [matches]);
};

export { Breakout, Center, Cluster, Cover, Frame, Grid, Position, Stack, useRefWidth };
//# sourceMappingURL=layout-blocks.esm.js.map
